import Vue from 'vue'

export default Vue.component('polaris-dropdown', {
    props: {
        list: {
            type: Array
        },
        triggerId: {
            type: String
        },
        label: {
            type: String
        },
        position: {
            type: String
        },
        arrow: {
            type: Boolean,
            default: true
        },
        arrowPosition: {
            type: String,
            default: 'center'
        },
        offset: {
            type: Number,
            default: 0
        }
    },
    data() {
        return {
            dropdownOpened: false,
        }
    },
    mounted() {
        window.onclick = () => {
            this.closeAll();
        };
        window.onscroll = () => {
            this.closeAll();
        };
        const trigger = document.getElementById(`${this.triggerId}`);
        if (trigger !== null) {
            trigger.onclick = (e) => {
                e.stopPropagation();
                this.closeAll();
                if (!this.dropdownOpened) {
                    this.open(e);
                } else {
                    this.close();
                }
            }
        }
    },
    methods: {
        closeAll() {
            const allDropdowns = document.querySelectorAll('.polaris-dropdown__menu');
            allDropdowns.forEach((dropdown) => {
                dropdown.classList.remove('open');
            });
        },
        open(e) {
            this.dropdownOpened = true;
            this.$refs.menu.classList.add('open');
            this.positioning(e);
        },
        close() {
            this.dropdownOpened = false;
            this.$refs.menu.classList.remove('open');
        },
        positioning(e) {
            /**
             *bottom: **,
             height: **,
             left: **,
             right: **,
             top: **,
             width: **,
             x: **,
             y: **,
             * @type {Object|ClientRect|*}
             * trigger coordinates
             */
            const coord = e.target.getBoundingClientRect();
            this.$refs.menu.style.top = `${coord.top + coord.height + this.offset}px`;
            this.$refs.menu.style.left = `${coord.left}px`;
            this.$refs.menu.style.maxWidth = `${coord.width}px`;
            switch(this.position) {
                case 'center':
                    this.$refs.menu.style.left = `${coord.left + coord.width / 2}px`;
                    this.$refs.menu.style.transform = 'translateX(-50%)';
                break;
                case 'right':
                    this.$refs.menu.style.left = `${coord.right - this.$refs.menu.clientWidth}px`;
                break;
                default:
                    this.$refs.menu.style.left = `${coord.left}px`;
            }
        }
    },
    render: function(h) {
        return h('div', {
            class: {
                [`polaris-dropdown`]: true,
            },
            on: {
                click: (e) => {
                    e.stopPropagation();
                }
            }
        }, [
            renderDropdownMenu(h, this)
        ])
    }
})

function renderDropdownMenu(h, comp) {
    return h('div', {
        class: {
            [`polaris-dropdown__menu`]: true,
        },
        ref: 'menu'
    }, [
        renderArrow(h, comp),
        renderList(h, comp)
    ])
}

function renderList(h, comp) {
    return h('ul', comp.list.map((item, index) => {
        return h('li', {
            class: {
                [`polaris-dropdown__menu__item`]: true,
                active: index === 0
            },
            on: {
                click: (e) => {
                    console.dir(e.target.parentElement.children);
                    const list = Array.from(e.target.parentElement.children);
                    list.forEach((li) => {
                        li.classList.remove('active');
                    });
                    e.target.classList.add('active');
                }
            }
        }, item[`${comp.label}`])
    }))
}

function renderArrow(h, comp) {
    return h('div', {
        class: {
            arrow: true,
            [`${comp.arrowPosition}`]: true
        },
    })
}